use crate::vec::Vec;

#[derive(Clone, Debug)]
pub struct BTree<T> {
    root: Node<T>,
    degree: usize,
}

#[derive(Clone, Debug)]
struct Node<T> {
    keys: Vec<T>,
    children: Vec<Node<T>>,
}

impl<T> Node<T>
where
    T: Ord,
{
    fn new(degree: usize, keys: Option<Vec<T>>, children: Option<Vec<Node<T>>>) -> Self {
        Node {
            keys: match keys {
                Some(keys) => keys,
                None => Vec::with_capacity(degree - 1),
            },
            children: match children {
                Some(children) => children,
                None => Vec::with_capacity(degree),
            },
        }
    }

    fn is_leaf(&self) -> bool {
        self.children.is_empty()
    }
}

impl<T: Ord> BTree<T> {
    pub fn new(branch_factor: usize) -> Self {
        let degree = 2 * branch_factor;
        Self {
            root: Node::new(degree, None, None),
            degree,
        }
    }

    pub fn clear(&mut self) {
        self.root = Node::new(self.degree, None, None);
    }

    fn rebalance() {
        todo!()
    }

    pub fn insert(&mut self, value: T) -> Option<T> {
        todo!()
    }

    pub fn has(&self, value: T) -> bool {
        todo!()
    }
}

#[cfg(test)]
mod test {
    use super::*;
    #[test]
    fn test_create() {
        let _tree = BTree::<u32>::new();
    }

    #[test]
    fn test_insert() {
        let mut tree = BTree::<u32>::new();
        let data = &[19, 125, 25, 16, 2, 73, 384, 435, 12924, 42, 125251, 2548];

        for d in data {
            assert!(tree.insert(*d).is_none())
        }

        for d in data {
            assert!(tree.has(*d))
        }
    }
}
